/*
    Copyright (C) 2002 Paul Davis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: gain_pan_meter.h,v 1.18 2004/05/14 22:15:43 pauld Exp $
*/

#ifndef __ardour_gtk_gain_pan_meter_h__
#define __ardour_gtk_gain_pan_meter_h__

#include <vector>
#include <gtk--.h>
#include <gtkmmext/slider_controller.h>
#include <gtkmmext/click_box.h>

#include "enums.h"

class Panner2d;

namespace ARDOUR {
	class IO;
	class Session;
}
namespace Gtkmmext {
	class FastMeter;
	class BarController;
}

class GainPanMeter : public Gtk::VBox
{
  public:
	GainPanMeter (ARDOUR::IO&, ARDOUR::Session&);
	~GainPanMeter ();

	void pan_changed (void *);

	void update_pan_sensitive ();
	void update_gain_sensitive ();

	void update_meters ();

	void effective_gain_display ();
	void effective_pan_display ();

	void set_width (Width);
	void setup_meters ();
	void setup_pan ();

	Gtk::HBox   pan_box;
	
	void set_meter_strip_name (string name);

  private:
	ARDOUR::IO& _io;
	ARDOUR::Session& _session;

	bool ignore_toggle;
	bool in_pan_update;

	Gtkmmext::VSliderController *gain_slider;
	Gtk::Adjustment              gain_adjustment;
	Gtk::Frame                   gain_display_frame;
	Gtkmmext::ClickBox           gain_display;
	Gtk::Frame                   peak_display_frame;
	Gtk::EventBox                peak_display;
	Gtk::Label                   peak_display_label;
	Gtk::Button                  gain_unit_button;
	Gtk::Label                   gain_unit_label;
	Gtk::HBox                    gain_display_box;
	Gtk::HBox                    fader_box;
	Gtk::DrawingArea             meter_metric_area;
	Gtk::ToggleButton            meter_pre_button;
        Gtk::Label                   meter_pre_label;
	Gtk::Table                   top_table;
	Width                       _width;

	gint meter_metrics_expose (GdkEventExpose *);

	static void _gain_printer (char buf[32], Gtk::Adjustment&, void *);
	void gain_printer (char buf[32], Gtk::Adjustment&);
	
	struct MeterInfo {
	    Gtkmmext::FastMeter *meter;
	    gint16          width;   
	    bool            packed;
	    
	    MeterInfo() { 
		    meter = 0;
		    width = 0;
		    packed = false;
	    }
	};

	static const guint16 regular_meter_width = 5;
	static const guint16 thin_meter_width = 2;
	vector<MeterInfo>    meters;
	float       max_peak;
	
	Panner2d*   panner;

	Gtk::VBox           pan_bar_packer;
	Gtk::Viewport       panning_viewport;
	Gtk::EventBox       panning_up;
	Gtk::Arrow          panning_up_arrow;
	Gtk::EventBox       panning_down;
	Gtk::Arrow          panning_down_arrow;
	Gtk::HBox           panning_scroll_box;
	Gtk::VBox           pan_vbox;

	gint panning_scroll_button_press_event (GdkEventButton*, int dir);
	gint panning_scroll_button_release_event (GdkEventButton*, int dir);
	
	Gtk::ToggleButton   panning_link_button;
	Gtk::Button         panning_link_direction_button;
	Gtk::HBox           panning_link_box;

	gint panning_link_button_press (GdkEventButton*);
	gint panning_link_button_release (GdkEventButton*);
	void panning_link_direction_clicked ();

	vector<Gtk::Adjustment*> pan_adjustments;
	vector<Gtkmmext::BarController*> pan_bars;

	void pan_adjustment_changed (unsigned long which);
	void pan_value_changed (unsigned long which);
	void pan_printer (char* buf, unsigned int, Gtk::Adjustment*);
	void update_pan_bars (bool only_if_aplay);
	void update_pan_linkage ();

	Gtk::HBox   hbox;
	Gtk::HBox   meter_packer;

	void gain_adjusted ();
	void gain_changed (void *);
	
	void meter_pre_toggled ();
	void meter_changed (void *);
	void gain_unit_changed ();
	void panner_changed ();
	
	void hide_all_meters ();
	void hide_pans ();

	void panner_moved (int which);
	void panner_bypass_toggled ();

	gint meter_button_press (GdkEventButton*, unsigned int);
	gint meter_button_release (GdkEventButton*, unsigned int);

	gint peak_button_release (GdkEventButton*);
	
	Gtk::Menu* meter_menu;
	void popup_meter_menu (GdkEventButton*);

	gint start_gain_touch (GdkEventButton*);
	gint end_gain_touch (GdkEventButton*);
	gint start_pan_touch (GdkEventButton*);
	gint end_pan_touch (GdkEventButton*);

	gint entry_focus_event (GdkEventFocus* ev);

	gint pan_button_event (GdkEventButton*, unsigned long which);
	Gtk::Menu* pan_menu;
	void build_pan_menu (unsigned long which);
	void pan_mute (unsigned long which);
	void pan_reset ();
};

#endif /* __ardour_gtk_gain_pan_meter_h__ */

